"""
    xkbregistry
    ~~~~~~~~~~~

    CFFI-based libxkbregistry bindings for Python.  See README.rst for details.

    :copyright: Copyright 2023 by Stephen Early <steve@assorted.org.uk>
    :license: MIT, see LICENSE for details.
"""
